<?php
/*
Plugin Name: AJAX Random Posts
Plugin URI: http://www.phoenixheart.net/2008/12/ajax-random-posts
Description: Displays a list of random posts using AJAX. Works with cache plugins, and highly customizable.
Version: 0.3.0.1
Author: Phoenixheart
Author URI: http://www.phoenixheart.net
*/

global $wpdb, $ajaxrp_plugin_name, $ajaxrp_plugin_dir, $ajaxrp_options;

$ajaxrp_plugin_name = 'AJAX Random Posts';
$ajaxrp_plugin_dir = get_settings('siteurl') . '/wp-content/plugins/ajax-random-posts/';
define('AJAXRP_CUSTOM_TAG', '<!--ajaxrp-->');

$ajaxrp_options = array(
	'ajaxrp_auto_show'			=> 1,
	'ajaxrp_before_html' 		=> '<h3>Random Posts</h3><div>',
	'ajaxrp_after_html'			=> '</div>',
	'ajaxrp_number_shown'		=> 10,
	'ajaxrp_loading_message'	=> '<p>Loading...</p>',
	'ajaxrp_post_format'		=> '{post} written by {author}'
);

load_plugin_textdomain($ajaxrp_text_domain);

function ajaxrp_install()
{
	global $ajaxrp_options;
	foreach ($ajaxrp_options as $key=>$value)
	{
		add_option($key, $value);
	}
}

register_activation_hook(__FILE__, 'ajaxrp_install');

/**
* @desc 	Includes the stylesheet and scripts
*/
function ajaxrp_head()
{
	global $ajaxrp_plugin_dir;
	$css_file = get_template_directory_uri() . 'ajaxrp-style.css';
	if (!file_exists($css_file))
	{
		$css_file = "{$ajaxrp_plugin_dir}css/ajaxrp-style.css";
	}
	
	printf('<link rel="stylesheet" type="text/css" media="screen" href="%s" />
	<script type="text/javascript" src="http://jqueryjs.googlecode.com/files/jquery-1.2.6.min.js"></script>
	<script type="text/javascript" src="%sjs/onload.js"></script>', $css_file, $ajaxrp_plugin_dir);
}

add_action('wp_head', 'ajaxrp_head');


function ajaxrp_prepare_post_data($post_content)
{	
	global $post;
	
	$data = array(
		'post_id'	=> $post->ID,
	);
	
	$placeholder = ajaxrp_get_placeholder($data);
	
	if (get_option('ajaxrp_auto_show'))
	{
		// remove the custom tag (if any) and add the place holder to the end of the post
		$post_content = str_replace(AJAXRP_CUSTOM_TAG, '', $post_content) . $placeholder;
		return $post_content; 
	}
	
	// replace the custom tag with the placeholder
	return str_replace(AJAXRP_CUSTOM_TAG, $placeholder, $post_content);
}

add_filter('the_content', 'ajaxrp_prepare_post_data', 5);

function ajaxrp_get_placeholder($data)
{
	// if this is not a single post, don't do anything
	if (!is_single())
	{
		return '';
	}
	
	$data = base64_encode(serialize($data));
	
	return sprintf('<!-- begin ajax random post data -->
	%s
	<div id="ajaxrp">
		%s
		<input type="hidden" name="ajaxrp_data" id="ajaxrpData" value="%s" />
	</div>
	%s
	<!-- end ajax random post data -->', 
	stripslashes(get_option('ajaxrp_before_html')), 
	stripslashes(get_option('ajaxrp_loading_message')), 
	$data,
	stripslashes(get_option('ajaxrp_after_html')));
}

function ajaxrp_request_handler()
{
	switch($_POST['ajaxrp_action'])
	{
		case 'get_posts':
			ajaxrp_get_posts();
			break;
		case 'options':
			ajaxrp_save_options();
			break;
		default:
			return false;
	}
	exit();
}

add_action('init', 'ajaxrp_request_handler', 5);

function ajaxrp_save_options()
{
	global $ajaxrp_options;
	
	foreach ($_POST as $key=>$value)
	{
		if ($key == 'ajaxrp_action' || !isset($ajaxrp_options[$key])) continue;
		if ($key == 'ajaxrp_number_shown')
		{
			$value = intval($value);
			if ($value < 1) $value = 10; // default number of posts is 10
		}
		update_option($key, $value);
	}
	header('Location: ' . get_bloginfo('wpurl') . '/wp-admin/options-general.php?page=ajax_random_posts.php&updated=true');
	die();
}

function ajaxrp_get_posts()
{
	$format = stripslashes(get_option('ajaxrp_post_format'));
	$data = unserialize(base64_decode($_POST['data']));
	$ret = '<ul>';
	
	query_posts(array(
		'showposts'		=> get_option('ajaxrp_number_shown'),
		'orderby'		=> 'rand',
		'post__not_in'	=>	array(intval($data['post_id'])),
	));
	
	while (have_posts())
	{
		the_post(); 

		$url = get_permalink();
		$title = get_the_title();
		$author_name = get_the_author();
		$author_url = get_the_author_url();
		$category_names = array();
		$category_links = array();

		foreach((get_the_category()) as $category)
		{
			$category_names[] = $category->cat_name;
			$category_links[] = sprintf('<a href="%s">%s</a>', get_category_link($category->cat_ID), $category->cat_name);
		}

		$tags = array(
			'{url}' 			=> $url,
			'{title}'			=> $title,
			'{post}'			=> sprintf('<a href="%s">%s</a>', $url, $title),
			'{categorynames}'	=> implode(', ', $category_names),
			'{categories}'		=> implode(', ', $category_links),
			'{authorname}'	 	=> $author_name,
			'{authorurl}'		=> $author_url,
			'{author}'			=> sprintf('<a href="%s">%s</a>', $author_url, $author_name),
			'{excerpt}'			=> apply_filters('the_excerpt', get_the_excerpt()),
			'{content}'			=> apply_filters('the_content', get_the_content()),
			'{date}'			=> the_date('', '', '', false),
			'{time}'			=> get_the_time(),
			'{tags}'			=> get_the_tag_list('', ', ', ''),
		);
		
		$ret .= str_replace(array_keys($tags), array_values($tags), "<li>$format</li>" . PHP_EOL);
	}
	
	$ret .= '</ul>';
	
	die($ret);
}

function ajaxrp_options_form()
{
	global $ajaxrp_plugin_dir;
	printf('
		<link rel="stylesheet" type="text/css" href="%scss/admin.css" />
		<script type="text/javascript" src="%sjs/admin-onload.js"></script>
		<div class="wrap">
		<form action="index.php" method="post" autocomplete="off">
			<p>
				<label>How many posts should the lists contain?<br />
				<input type="text" value="%s" name="ajaxrp_number_shown" class="required" style="width: 40px" />
			</p>
			<p><label>Automatically display the random posts in a single post page?</label></p>
			<p class="toggler">Get help with this</p>
			<div class="toggled" style="font-size: 11px; display: none; background: #ffffef; padding: 5px 10px">
				If you turn this to "No", there are still 2 ways to display a list of random posts:
				<ol>
					<li>Manually put a custom tag <strong>&lt;!--ajaxrp--&gt;</strong> at the end of the post you want the list to follow up, or</li>
					<li>Place this php code: <br />
						<strong>&lt;?php if (function_exists(\'ajaxrp\')) ajaxrp(); ?&gt;</strong><br />
						at a proper place in your template\'s <strong>single.php</strong> page</li>
				</ol>
			</div>
			<p style="margin-top: -5px">
				<select name="ajaxrp_auto_show">
					<option value="1">Yes</option>
					<option value="0" %s>No</option>
				</select>
			</p>
			<p>
				<label>The HTML code before the list<br />
				<textarea name="ajaxrp_before_html" style="width: 400px">%s</textarea>
			</p>
			<p>
				<label>The HTML code after the list<br />
				<textarea name="ajaxrp_after_html" style="width: 400px">%s</textarea>
			</p>
			<p>
				<label>The "loading" message<br />
				<textarea name="ajaxrp_loading_message" style="width: 400px">%s</textarea>
			</p>
			<p><label>Format of each item in the list </label></p>
			<p class="toggler">Get help with this</p>
			<div class="toggled" style="font-size: 11px; display: none; background: #ffffef; padding: 5px 10px">
				Available tags:<br />
				<ul>
					<li><strong>{url}</strong> - The permalink to the post</li>
					<li><strong>{title}</strong> - The title of the post</li>
					<li><strong>{post}</strong> - Link to the post. A combination of {url} and {title}</li>
					<li><strong>{categorynames}</strong> - Names of the categories, seperated by commas</li>
					<li><strong>{categories}</strong> - Links to the categories, seperated by commas</li>
					<li><strong>{authorname}</strong> - Name of the post\'s author</li>
					<li><strong>{authorurl}</strong> - The URL of the author</li>
					<li><strong>{author}</strong> - Link to the author. A combination of {authorurl} and {authorname}</li>
					<li><strong>{excerpt}</strong> - The post\'s excerpt (short summary)</li>
					<li><strong>{content}</strong> - The post\'s full-length content</li>
					<li><strong>{date}</strong> - The post\'s date</li>
					<li><strong>{time}</strong> - The post\'s time</li>
					<li><strong>{tags}</strong> - A comma-seperated list of this post\'s (clickable) tags</li>
				</ul>
			</div>
			<p style="margin-top: -5px">
				<textarea name="ajaxrp_post_format" class="required" style="width: 400px">%s</textarea>
				<input type="hidden" value="options" name="ajaxrp_action" />
			</p>
			<div class="submit">
				<input type="submit" value="Save Options" />
			</div>
		</form>
	</div>',
	$ajaxrp_plugin_dir,
	$ajaxrp_plugin_dir,
	get_option('ajaxrp_number_shown'),
	get_option('ajaxrp_auto_show') == 0 ? 'selected="selected"' : '',
	stripslashes(get_option('ajaxrp_before_html')),
	stripslashes(get_option('ajaxrp_after_html')),
	stripslashes(get_option('ajaxrp_loading_message')),
	stripslashes(get_option('ajaxrp_post_format')));
}

/**
 * @desc	Adds the Options menu item
 */
function ajaxrp_menu_items()
{
	add_options_page('AJAX Random Posts', 'AJAX Random Posts', 8, basename(__FILE__), 'ajaxrp_options_form');
}

# Hook into the settings menu
add_action('admin_menu', 'ajaxrp_menu_items');

function ajaxrp()
{
	global $post;
	
	$data = array(
		'post_id'	=> $post->ID,
	);
	
	echo ajaxrp_get_placeholder($data);
}









